/**************************************************************************************
* CFMove - a program that moves/copies files inside of the CF card
* of a TRGpro
* Copyright (C) 2001 by Enno Welbers
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or 
* (at your opinion) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the program, see LICENSE.txt; if not, write to the Free
* Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  0211-1307  USA
*
* You can contact the author via: mkernel@users.sourceforge.net or enno_welbers@gmx.de
**************************************************************************************/
#include "cfmove.h"

UInt16 FfsLibRef;
char *src="", *dest="";
MemHandle srcH,destH;
cldata client;
int state;

#define BUF_SIZE        40960L
void Core(int mvcp)
{
    UInt16         num_read;
    UInt8	  *buf;
    Int16          src_fd, dest_fd;
    bool	   failed=false;
    coordinates    point;
    stat           filestatistic;
    int            expand=1,portions,count=0;
    /* create a polyogon for the progress meter */
    MoveRoot();
    point.x=29;
    point.y=30;
    addpoint(&client,3,0,1,true,point);
    point.y=point.y+10;
    addpoint(&client,3,1,1,true,point);
    point.x=131;
    addpoint(&client,3,2,1,true,point);
    point.y=point.y-10;
    addpoint(&client,3,3,1,true,point);
    /* draw the polygon */
    drawpoly(&client,3);
    
    /* allocate a 40K buffer to hold data */
    if ((buf = (UInt8 *)MemPtrNew(BUF_SIZE)) == NULL)
    {
        FrmCustomAlert(1000,"Unable to create buffer for copying, exiting...","","");
        return;
    }
    if ((src_fd = FfsOpen(FfsLibRef,src,O_RDONLY,0)) == -1)
    {
    	FrmCustomAlert(1000,"Unable to open source file, exiting...","","");
        count=FfsGetErrno(FfsLibRef);
        MemPtrFree(buf);
        return;
    }
    /*analyse the source file and get how much one percent is */
    FfsFstat(FfsLibRef,src_fd,&filestatistic);
    portions=filestatistic.st_size/40960L;
    if(portions==0)portions=1;
    expand=100/portions;
    
    if ((dest_fd = FfsOpen(FfsLibRef,dest,O_CREAT|O_WRONLY,S_IWRITE)) == -1)
    {
    	FrmCustomAlert(1000,"Unable to create destination file, exiting...","","");
        FfsClose(FfsLibRef,src_fd);
        MemPtrFree(buf);
        return;
    }
    point.x=30;
    point.y=31;
    
    /* copy the data */
    while(!FfsEof(FfsLibRef, src_fd))
    {
        if ((num_read = FfsRead(FfsLibRef,src_fd,buf,(Int16)BUF_SIZE)) == 0xFFFF)
        {
            FrmCustomAlert(1000,"Unable to read from CF card to buffer, exiting...","","");
            failed=true;
            break;
        }

        if (num_read != FfsWrite(FfsLibRef,dest_fd,buf,num_read))
        {
	    FrmCustomAlert(1000,"Unable to read from buffer to CF card, exiting...","","");
	    failed=true;
            break;
        }
        for(count=0;count<=expand;count++)
          if(point.x+count<130)
            line(&client,1,point.x+count,point.y,point.x+count,39);
        point.x=point.x+expand;
    }
    FfsClose(FfsLibRef,src_fd);
    FfsClose(FfsLibRef,dest_fd);
    if(failed==true)
      {
        FrmCustomAlert(1000,"Copy/move failed, deleting destination file and keeping source...","","");
        FfsRemove(FfsLibRef,dest);
      }
    else if(mvcp==2)
      FfsRemove(FfsLibRef,src);
    MemPtrFree(buf);
}

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 Launchflags)
{
  int chosen;
  bool loopy=true;
  char *srcfn;
  MemHandle srcfnH;
  int count=0;
  
  if(cmd!=sysAppLaunchCmdNormalLaunch)return;
  InitLibs();
  title("please wait...");
  MoveRoot();
  processDir();
  title("SRC?");
  state=SRC;
  chosen=UIDirContent();
  if(chosen==-2){shutdown(false,false,true,true);return;}
  if(chosen>dircontentmax)
    {
      title("SRC? (First Value impossible)");
      chosen=-1;
    }
  if(DirContent[chosen].isdir==false)
    loopy=false;
  while(loopy==true)
    {
      if(chosen!=-1)
        {
          SwitchtoDir(chosen);
          freeprocessvars();
          processDir();
        }
      chosen=UIDirContent();
      if(chosen==-2){shutdown(false,false,true,true);return;}
      if(chosen>dircontentmax)
        {
          title("SRC? (First Value impossible)");
          chosen=-1;
          loopy=true;
        }
      else if(DirContent[chosen].isdir==true)
        loopy=true;
      else 
        loopy=false;
    }
  fillsrc(chosen);
  srcfnH=MemHandleNew(StrLen(DirContent[chosen].name)+1);
  srcfn=MemHandleLock(srcfnH);
  while(DirContent[chosen].name[count]!='\0')
    {
      srcfn[count]=DirContent[chosen].name[count];
      count++;
    }
  srcfn[count]='\0';
  freeprocessvars();
  MoveRoot();
  title("DEST? (Use OK (Circle))");
  state=DEST;
  processDir();
  chosen=UIDirContent();
  if(chosen==-2)return;
  if(chosen>dircontentmax)
    if(newfile(srcfn)==-1){MemHandleUnlock(srcfnH);MemHandleFree(srcfnH);shutdown(true,false,true,true);return;}
  if(chosen<=dircontentmax)
  while(DirContent[chosen].isdir==true)
    {
      SwitchtoDir(chosen);
      freeprocessvars();
      processDir();
      chosen=UIDirContent();
      if(chosen==-2){MemHandleUnlock(srcfnH);MemHandleFree(srcfnH);shutdown(true,false,true,true);return;}
      if(chosen>dircontentmax)
        {
          if(newfile(srcfn)==-1){MemHandleUnlock(srcfnH);MemHandleFree(srcfnH);shutdown(true,false,true,true);return;}
          break;
        }
    }
  chosen=MvCpCa();
  freeprocessvars();
  if(chosen!=0)
    Core(chosen);
  MemHandleUnlock(srcfnH);
  MemHandleFree(srcfnH);
  shutdown(true,true,false,true);
}

void fillsrc(int chosen)
{
  int count,length=1,count2;
  char *cwd;
  bool info;
  title("please wait, computing full src filename...");
  while(FfsGetcwd(FfsLibRef,NULL,length)==NULL)
    length++;
  cwd=FfsGetcwd(FfsLibRef,NULL,length);
  if(cwd[0]=='\\' && cwd[1]=='\0')
    info=true;
  else
    info=false;
  srcH=MemHandleNew(StrLen(cwd)+StrLen(DirContent[chosen].name));
  src=MemHandleLock(srcH);
  count=0;
  if(info==false)
    while(count<StrLen(cwd))
      {
        src[count]=cwd[count];
        count++;
      }
  count2=0;
  while(count2<StrLen(DirContent[chosen].name))
    {
      src[count]=DirContent[chosen].name[count2];
      count++;
      count2++;
    }
  src[count]='\0';
}

bool InitLibs(void)
{
  Err err;
  err=SysLibFind(FfsLibName,&FfsLibRef);
  if(err!=0)
    {
      if(err=SysLibLoad(FfsLibTypeID,FfsLibCreatorID,&FfsLibRef)!=0)
        return false;
    }
  initengine(&client);
  initptr();
  err=FfsLibOpen(FfsLibRef);
  readpolyfromdisk(&client,1,0,"CFMove UI 2");
  readpolyfromdisk(&client,2,0,"CFMove UI 1");
  return true;
}
  
void DeInitLibs(void)
{
  if(FfsLibClose(FfsLibRef)==0)
    SysLibRemove(FfsLibRef);
  deinitengine(&client);
}

int newfile(char *srcfn)
{
  int count=0,length=1,count2,size=1,lastpos=-1;
  char *cwd;
  char *fn;
  MemHandle fnH;
  event Event;
  coordinates point;
  FieldType *Field;
  void *FormP;
  FormType *FormP2;

  FormP=client.Form;
  size=StrLen(srcfn)+1;
  fnH=MemHandleNew(257);
  fn=MemHandleLock(fnH);
  while(srcfn[count]!='\0')
    {
      fn[count]=srcfn[count];
      count++;
    }
  fn[count]='\0';
  MemHandleUnlock(fnH);
  Field=FldNewField(&FormP,1001,30,40,100,12,stdFont,256,true,true,true,false,leftAlign,false,false,false);
  client.Form=FormP;
  FldSetText(Field,fnH,0,257);
  FldDrawField(Field);
  FrmSetFocus(client.Form,FrmGetObjectIndex(client.Form,1001));  
  point.x=42;point.y=119;
  drawtext(&client,1,point,false,0,"1");
  point.y=142;
  drawtext(&client,1,point,false,0,"2");
  point.x=22;
  point.x=80;
  point.y=113;
  drawtext(&client,1,point,false,0,"1: OK");
  point.y=123;
  drawtext(&client,1,point,false,0,"2: CANCEL");
  lastpos=count-1;
  title("Enter filename");
  while(1==1)
    {
      Event.type=0;
      getevent(&client,&Event);
      if(client.OSEvent.screenY<52|client.OSEvent.eType!=penDownEvent)
      FldHandleEvent(Field,&client.OSEvent);
      if(Event.type==EventAppStopEvent)
      {
        MemHandleFree(fnH);
        FormP2=client.Form;
        FldEraseField(Field);
        FrmRemoveObject(&FormP2,FrmGetObjectIndex(FormP,1001));
        client.Form=FormP2;
        return -1;
      }
      else if(Event.type==EventMouseButtonPressEvent)
        {
          if(Event.ebtn.x>42 && Event.ebtn.y>113 && Event.ebtn.y<130 && Event.ebtn.x<58)
            break;
          else if(Event.ebtn.x>42 && Event.ebtn.y>140 && Event.ebtn.y<158 && Event.ebtn.x<58)
            {
              MemHandleFree(fnH);
              FldEraseField(Field);
              FormP2=client.Form;
              FrmRemoveObject(&FormP2,FrmGetObjectIndex(FormP,1001));
              client.Form=FormP2;
              return -1;
            }
        }
    }
  point.x=42;point.y=119;
  removetext(&client,point,false,"1");
  point.y=142;
  removetext(&client,point,false,"2");
  point.x=80;
  point.y=113;
  removetext(&client,point,false,"1: OK");
  point.y=123;
  removetext(&client,point,false,"2: CANCEL");

  title("please wait, computing full path...");
  while(FfsGetcwd(FfsLibRef,NULL,length)==NULL)
    length++;
  cwd=FfsGetcwd(FfsLibRef,NULL,length);
  destH=MemHandleNew(StrLen(cwd)+StrLen(fn));
  dest=MemHandleLock(destH);
  fnH=FldGetTextHandle(Field);
  fn=MemHandleLock(fnH);
  count=0;
  while(count<StrLen(cwd))
    {
      dest[count]=cwd[count];
      count++;
    }
  count2=0;
  while(count2<StrLen(fn))
    {
      dest[count]=fn[count2];
      count++;
      count2++;
    }
  dest[count]='\0';
  MemHandleUnlock(fnH);
  MemHandleFree(fnH);
  FldEraseField(Field);
  FormP2=client.Form;
  FrmRemoveObject(&FormP2,FrmGetObjectIndex(FormP,1001));
  client.Form=FormP2;
  return 0;
}

void playsnd(void)
{
  UInt32 volume,ticks,ticks2;
  SndCommandType sndcmd;
  volume=PrefGetPreference(prefSysSoundVolume);
  sndcmd.cmd=sndCmdFreqDurationAmp;
  sndcmd.param1=1500;
  sndcmd.param2=50;
  sndcmd.param3=volume;
  SndDoCmd(NULL,&sndcmd,0);
  ticks=ticks2=0;
  ticks=ticks2=TimGetTicks();
  while(ticks2-ticks<10)
    ticks2=TimGetTicks();
  SndDoCmd(NULL,&sndcmd,0);
  ticks=ticks2=0;
  ticks=ticks2=TimGetTicks();
  while(ticks2-ticks<10)
  ticks2=TimGetTicks();
  sndcmd.param1=1000;
  sndcmd.param2=100;
  SndDoCmd(NULL,&sndcmd,0);
}

void playsnd2(void)
{
  UInt32 volume,ticks,ticks2;
  SndCommandType sndcmd;
  volume=PrefGetPreference(prefSysSoundVolume);
  sndcmd.cmd=sndCmdFreqDurationAmp;
  sndcmd.param1=1500;
  sndcmd.param2=50;
  sndcmd.param3=volume;
  SndDoCmd(NULL,&sndcmd,0);
}

void shutdown(bool srcb, bool destb, bool freeDirContent, bool deinitlibs)
{
  if(srcb==true)
    {
      MemHandleUnlock(srcH);
      MemHandleFree(srcH);
    }
  if(destb==true)
    {
      MemHandleUnlock(destH);
      MemHandleFree(destH);
    }
  if(freeDirContent==true)
    freeprocessvars();
  if(deinitlibs==true)
    DeInitLibs();
}
